package sun.font;

import java.util.Iterator;
import java.util.Map;
import java.security.AccessController;
import sun.misc.ThreadGroupUtils;
import java.util.HashMap;
import java.io.OutputStream;
import java.io.File;
import java.util.concurrent.TimeUnit;
import sun.awt.AppContext;
import java.util.concurrent.Semaphore;

public class CreatedFontTracker
{
    public static final int MAX_FILE_SIZE = 33554432;
    public static final int MAX_TOTAL_BYTES = 335544320;
    static CreatedFontTracker tracker;
    int numBytes;
    
    public static synchronized CreatedFontTracker getTracker() {
        if (CreatedFontTracker.tracker == null) {
            CreatedFontTracker.tracker = new CreatedFontTracker();
        }
        return CreatedFontTracker.tracker;
    }
    
    private CreatedFontTracker() {
        this.numBytes = 0;
    }
    
    public synchronized int getNumBytes() {
        return this.numBytes;
    }
    
    public synchronized void addBytes(int n) {
        this.numBytes += n;
    }
    
    public synchronized void subBytes(int n) {
        this.numBytes -= n;
    }
    
    private static synchronized Semaphore getCS() {
        AppContext appContext = AppContext.getAppContext();
        Semaphore value = (Semaphore)appContext.get(CreatedFontTracker.class);
        if (value == null) {
            value = new Semaphore(5, true);
            appContext.put(CreatedFontTracker.class, value);
            return value;
        }
        return value;
    }
    
    public boolean acquirePermit() throws InterruptedException {
        return getCS().tryAcquire(120L, TimeUnit.SECONDS);
    }
    
    public void releasePermit() {
        getCS().release();
    }
    
    public void add(File file) {
        CreatedFontTracker.TempFileDeletionHook.add(file);
    }
    
    public void set(File file, OutputStream outputStream) {
        CreatedFontTracker.TempFileDeletionHook.set(file, outputStream);
    }
    
    public void remove(File file) {
        CreatedFontTracker.TempFileDeletionHook.remove(file);
    }
    
    private static class TempFileDeletionHook
    {
        private static HashMap<File, OutputStream> files;
        private static Thread t;
        
        static void init() {
            if (CreatedFontTracker.TempFileDeletionHook.t == null) {
                AccessController.doPrivileged(()/*  */ -> {
                    (CreatedFontTracker.TempFileDeletionHook.t = new Thread(ThreadGroupUtils.getRootThreadGroup(), CreatedFontTracker.TempFileDeletionHook::runHooks)).setContextClassLoader(null);
                    Runtime.getRuntime().addShutdownHook(CreatedFontTracker.TempFileDeletionHook.t);
                    return null;
                });
            }
        }
        
        static synchronized void add(File key) {
            init();
            CreatedFontTracker.TempFileDeletionHook.files.put(key, null);
        }
        
        static synchronized void set(File key, OutputStream value) {
            CreatedFontTracker.TempFileDeletionHook.files.put(key, value);
        }
        
        static synchronized void remove(File key) {
            CreatedFontTracker.TempFileDeletionHook.files.remove(key);
        }
        
        static synchronized void runHooks() {
            if (CreatedFontTracker.TempFileDeletionHook.files.isEmpty()) {
                return;
            }
            for (Map.Entry entry : CreatedFontTracker.TempFileDeletionHook.files.entrySet()) {
                try {
                    if (entry.getValue() != null) {
                        ((OutputStream)entry.getValue()).close();
                    }
                }
                catch (Exception ex) {}
                ((File)entry.getKey()).delete();
            }
            return;
        }
        
        static {
            CreatedFontTracker.TempFileDeletionHook.files = new HashMap();
            CreatedFontTracker.TempFileDeletionHook.t = null;
        }
    }
}
